package FFRIndexed::File;
use strict;
use URI::file;
use Data::Dumper qw(Dumper);
use base 'Table::Denormalized::Row';
#use File::Basename;
use overload
  '""' => sub { shift->{uri} },
  fallback => 1,
  ;

=head1 NAME

FFRIndexed::File - entry for a ressource

=head1 SYNOPSIS

  my $idx = FFRIndexed->new();
  my $resultset = $idx->query({ mime_type => 'audio/mpeg' });

  for my $mp3 (@$resultset) {
    printf "%s\t%s\t%s\n", $mp3->artist, $mp3->track, $mp3->title;
  };

=cut

use vars '$AUTOLOAD';
use vars qw($VERSION);
$VERSION = '0.06';

=head1 CAVEATS

Since this package implements the accessors for all hash entries
via C<$AUTOLOAD>, the usual caveats about inherited C<$AUTOLOAD>
apply.

This package is a bastardized, simplified version of L<Class::DBI>, for
a very specialized purpose. In the far future, this should be
unified with the relevant parts of L<Class::DBI> instead of reimplementing
it.

=cut

sub AUTOLOAD {
  my ($method) = ($AUTOLOAD =~ /::([^:]+)$/g);
  my $class = ref $_[0];

  {
    no strict 'refs';
    *{ "$class\::$method" }
      = sub {   exists $_[0]->{$method}    ? $_[0]->{$method}
              : exists $_[0]->{"_$method"} ? $_[0]->{"_$method"}
              : die "No entry '$method' in $_[0]" };
    goto &{ "$class\::$method" };
  };
};

=head2 C<< $f->filename >>

Returns the filename including the full path

=cut

sub filename { URI->new($_[0]->uri)->file };
#sub basename { basename( sprintf "%s", $_[0]->filename )};
sub basename { warn $_[0]->filename; $_[0]->filename };

sub file_object { IO::Handle::Lazy->new( shift->filename ) };

=head2 C<< $file->stale >>

Returns true if the on-disk and in-database data for
the file differ. This should be relatively fast as "only"
a C<stat> call and a few reads of the file are done.
The C<md5> (for example) is not recalculated.

=cut

sub stale {
  my ($self) = @_;
  $self->{_index}->stale($self)
};

=head2 C<< $file->rescan >>

Rescans the information from the on-disk storage
into the database. This will take some time.

=cut

sub rescan {
  my ($self) = @_;
  #warn "Rescanning " . $self->filename;
  my $data = $self->{_index}->get_file_info($self->filename);

  my @new = (%$data, $self->_skeleton);
  %$self = @new;
  $self->update;

  $self;
};

=head2 C<< $file->refresh >>

Convenience method. Checks the row for stale-ness and rescans the data if it is
found to be stale.

=cut

sub refresh {
  my ($self) = @_;
  $self->rescan
    if $self->stale;
};

=head2 C<< $file->unlink >>

Deletes the file from disk I<and> from the database.

=cut

sub unlink {
  my ($self) = @_;
  $self->delete;
  unlink $self->filename;
};

# Needed because we have an $AUTOLOAD handler
sub DESTROY {};

=head1 SEE ALSO

L<Table::Denormalized::Row>, another bastardized version of what L<Class::DBI> provides.

=cut

1;
