package Table::Denormalized::ColumnRefiner;
use strict;
use base 'Class::Accessor';
use Table::Denormalized;
use Table::ColumnRefinement;
use Carp qw(croak);
use vars qw($VERSION);
$VERSION = '0.06';

__PACKAGE__->mk_accessors(qw(table columns));

=head1 NAME

Table::Denormalized::ColumnRefiner - find columns that refine a query

=cut

sub new {
  my ($package,%args) = @_;
  croak "No table given to refine"
    unless $args{table};
  $args{columns} ||= [ sort $args{table}->columns ];
  $package->SUPER::new(\%args);
};

=head2 C<< $r->refinements QUERY, RESULTCOUNT >>

Returns the column refinements.

The (optional) result count is the number of rows in the result
of the query. If this is unknown, it can be left out.
A modified version of the query will then be run for the row count.

=cut

sub refinements {
  my ($self,$query,$count) = @_;

  my $table = $self->table;

  if (! defined $count) {
    # delete limit from query
    # make count(*) the select
    my ($sql,@bind) = $query;
    $count = 255;
  };

  grep { my $t = $_->is_trivial; ! $t }
    map {Table::ColumnRefinement->new($table,$_,$count,$query)} (@{$self->columns});
};

=head2 C<< $r->implicit_columns QUERY, RESULTCOUNT >>

Returns the implicit columns (and their values).

=cut

sub implicit_columns {
  my ($self,$query,$count) = @_;

  my $table = $self->table;

  if (! defined $count) {
    # delete limit from query
    # make count(*) the select
    my ($sql,@bind) = $query;
    $count = 255;
  };

  grep { $_->is_implicit; }
    map {Table::ColumnRefinement->new($table,$_,$count,$query)} (@{$self->columns});
};

1;
