package Table::Denormalized::ResultSet;
use strict;
use Storable qw(dclone); # We might want Clone.pm instead, which might be faster/different
use Carp qw(croak);
use Data::Dumper;
use vars qw($VERSION);
$VERSION = '0.06';

my %query; # inside-out object

sub new {
  my ($class,$ref,$query) = @_;
  bless $ref, $class;
  $query{$ref} = $query;
  $ref;
};

sub DESTROY { delete $query{$_[0]} };

sub query { $query{$_[0]} };
sub results { @{$_[0]} };

=head2 C<< $resultset->refine $query >>

Returns a new query which is a modification of the
current query. The existing query is augmented in the following
way, depending on the type of C<$query> :

If C<$query> is a hashref, it is added as an AND restriction.

If C<$query> is an arrayref, it is added as a local OR restriction
(B<which is not implemented yet>).

=cut

sub refine {
  my ($self,$refinement) = @_;

  if (ref $refinement eq 'HASH') {
    my $res;
    my $q = $self->query;
    if (ref $q eq 'HASH') {
      $res = dclone($q);
      @$res{ keys %$refinement } = values %$refinement;
    } elsif (ref $q eq 'ARRAY') {
      if (@$q == 0) {
        $res = dclone($refinement);
      } else {
        $res = [ -and => { -nest => dclone $q }, { -nest => $refinement } ]
      };
    };

    return $res;
  } else {
    croak sprintf "Unknown reftype '%s', I can only handle 'HASH'", (ref $refinement || "no reference");
  };
};

1;

=head1 AUTHOR

Max Maischein, E<lt>corion@cpan.orgE<gt>

=head1 SEE ALSO

L<DBI>, L<Class::DBI>
