#!/usr/bin/perl -w
use strict;
use Data::Dumper;
use lib 'lib';
use Table::Denormalized;
use lib 'inc';
use TableLoader;

my (@tests,@where);
BEGIN {
  @tests = (
  ['', ['uri'], [], []],

  ['foo', ['uri'], [], []],
  ['foo%', ['uri'], [], [1]],
  ['%foo%', ['uri'], [], [1]],

  ['foo', ['category'], [], []],
  ['foo%', ['category'], [], [3]],
  ['%foo%', ['category'], [], [3]],

  ['%foo%', ['uri','category'], [], [1,3]],

  ['dup%', ['uri','category'], [], [4]],
)};
use Test::More tests => (scalar @tests)*3;

my $table = create_and_load("test",\*DATA);
my $primary = $table->primary;
for (@tests) {
  my ($value,$columns,$where,$expected_rows) = @$_;
  my $name = sprintf "'%s' in (%s)", $value, join ",", @$columns;

  my $results = $table->query_ex(
    columns => [$primary],
    super_search => $value,
    super_search_columns => $columns,
    where => $where,
  );
  isa_ok( $results, 'Table::Denormalized::ResultSet' );

  my @results = map {$_->{_id}} @$results;

  my %uniq;
  for (@results) {
    $uniq{$_}++;
  };

  local $" = ",";
  is_deeply([ grep { $uniq{$_} > 1 } sort {$a <=> $b} keys %uniq],
            [],
            "No duplicate rows are returned for $name"
  );

  is_deeply(\@results,$expected_rows,"$name returns the expected rows");
};

__DATA__
uri:VARCHAR(20),category:VARCHAR(20),location:VARCHAR(20),year:INTEGER
foo.jpg,people,braga,2005
bar.jpg,people,frankfurt,2005
baz.jpg,food,frankfurt,2004
dup.jpg,dupe,braga,2005